/******************************************************************************
Author:         David Szybala
Website:        www.davidszybala.com
Alt Website:   www.twistedgerbil.com
Program:        selector.ms
Date:            05/02/10
Description:    This script will select objects based on certain criteria.
******************************************************************************/

try (destroyDialog select_object_rollout) catch()


struct ObjectSelector
(
    temp = #(),
    
    /**************************************************************************
    Returns a sorted list of all material class names
    **************************************************************************/
    fn get_material_classes =
    (
        local class_names = for current in material.classes collect (current as string)
        sort class_names
        
        class_names
    ), --end fn
    
    /**************************************************************************
    Returns a sorted list of all modifier class names
    **************************************************************************/
    fn get_modifier_classes =
    (
        local class_names = for current in modifier.classes collect (current as string)
        sort class_names
        
        class_names
    ), --end fn
    
    /**************************************************************************
    Returns all objects with a material of type 'class_name'
    **************************************************************************/
    fn get_by_material class_name =
    (
        local matches = #()
        
        for current in objects do
        (
            if current.material != undefined do
            (
                if ((classOf current.material) as string) == class_name do
                    append matches current
            ) --end if
        ) --end for
        
        matches --return value
    ), --end fn
    
    /**************************************************************************
    Returns all objects with a modifier of type 'class_name'
    **************************************************************************/
    fn get_by_modifier class_name =
    (
        local matches = #()
        
        for current in objects do
        (
            if current.modifiers.count > 0 do
            (
                loop_control = true
                
                for i = 1 to current.modifiers.count while loop_control do
                (
                    if ((classOf current.modifiers[i]) as string) == class_name do
                    (
                        append matches current
                        loop_control = false
                    ) --end if
                ) --end for
            ) --end if
        ) --end for
        
        matches --return value
    ), --end fn
    
    /**************************************************************************
    Recursive function. Collects all parents and grandparents of the current object
    **************************************************************************/
    fn get_all_parents current_object =
    (
        if current_object.parent != undefined do
        (
            append this.temp current_object.parent
            this.get_all_parents current_object.parent
        ) --end if
    ), --end fn
    
    /**************************************************************************
    Returns the parents of the current selection
    **************************************************************************/
    fn get_parents include_extended =
    (
        local matches = for current in selection where current.parent != undefined collect current.parent
        
        if include_extended do
        (
            --CRITICAL! Clear previous data!
            free this.temp
            
            for current in matches do
                this.get_all_parents current
            
            join matches this.temp
        ) --end if
        
        makeUniqueArray matches --return value
    ), --end fn
    
    /**************************************************************************
    Recursive function. Collects all children and grandchildren of the current object
    **************************************************************************/
    fn get_all_children current_object =
    (
        if current_object.children.count > 0 do
        (
            join this.temp current_object.children
            
            for child in current_object.children do
                this.get_all_children child
        ) --end if
    ), --end fn
    
    /**************************************************************************
    Returns the parents of the current selection
    **************************************************************************/
    fn get_children include_extended =
    (
        local matches = #()
        
        for current in selection do
        (
            if current.children.count > 0 do
                join matches current.children
        ) --end for
        
        if include_extended do
        (
            --CRITICAL! Clear previous data!
            free this.temp
            
            for current in matches do
                this.get_all_children current
            
            join matches this.temp
        ) --end if
        
        makeUniqueArray matches --return value
    ), --end fn
    
    /**************************************************************************
    Returns all objects with animation (keyframes/wires)
    **************************************************************************/
    fn get_by_animation =
    (
        local matches = #()
        
        for current in objects do
        (
            if current.isAnimated do
                append matches current
        ) --end for
        
        matches --return value
    ), --end fn
    
    /**************************************************************************
    Returns all geometry with no UVs
    **************************************************************************/
    fn get_no_uv_geometry =
    (
        local matches = #()
        local total_vertices = 0
    
        for current in geometry do
        (
            if (classOf current != boneGeometry) and (classOf current != targetObject) do
            (
                total_vertices = 0
                
                for i = 1 to meshop.getNumMaps current.mesh do
                (
                    try
                    (
                        total_vertices += meshop.getNumMapVerts current.mesh i
                    ) --end try
                    catch()
                ) --end for
                
                if total_vertices == 0 do
                    append matches current
            ) --end if
        ) --end for
            
        matches --return value
    ), --end fn
    
    /**************************************************************************
    Returns all objects from 'object_list' that are not hidden
    **************************************************************************/
    fn get_visible_objects object_list =
    (
        local visible_objects = for current in object_list where not current.isHidden collect current
        
        visible_objects --return value
    ) --end fn
) --end struct


(--put script into local scope
    global select_object_rollout
    local select_objects_ini = getDir #plugcfg + "\\Selector.ini"
    local select_objects_position
    
    rollout select_object_rollout "Selector"
    (
        local selector = ObjectSelector()
        
        groupBox material_group "Material" pos:[5,5] width:240 height:70
            dropdownlist material_dropdown pos:[15,22] width:220 height:20
            button select_material_button "Select Objects by Material" pos:[15,50] width:220 height:20
        
        groupBox modifier_group "Modifier" pos:[5,80] width:240 height:70
            dropdownlist modifier_dropdown pos:[15,97] width:220 height:20
            button select_modifier_button "Select Objects by Modifier" pos:[15,125] width:220 height:20
        
        groupbox hierarchy_group "Hierarchy" pos:[5,155] width:240 height:90
            checkbox parent_checkbox "Parents" pos:[15,175] width:60 height:15
            checkbox extended_parent_checkbox "Include Extended Parents" pos:[90,175] width:150 height:15 enabled:false
            checkbox children_checkbox "Children" pos:[15,195] width:60 height:15 checked:true
            checkbox extended_children_checkbox "Include Extended Children" pos:[90,195] width:150 height:15
            button select_hierarchy_button "Select Hierarchy" pos:[15,220] width:220 height:20
        
        groupbox misc_group "Misc" pos:[5,250] width:240 height:70
            button select_animation_button "Select Objects with Keyframes/Wires" pos:[15,270] width:220 height:20
            button select_no_uv_button "Select Geometry with No UVs" pos:[15,295] width:220 height:20
        
        groupbox option_group "Options" pos:[5,325] width:240 height:60
            checkbox add_selection_checkbox "Add to Selection" pos:[15,345] width:110 height:15
            checkbox visible_only_checkbox "Visible Objects Only" pos:[15,365] width:120 height:15
        
        label status_label "INFO: Standing by..." pos:[5,395] width:240 height:15
        
        /**********************************************************************
        UI open event handler. Prepares the UI for first time open
        **********************************************************************/
        on select_object_rollout open do
        (
            material_dropdown.items = selector.get_material_classes()
            modifier_dropdown.items = selector.get_modifier_classes()
        ) --end on
        
        /**********************************************************************
        UI move event handler. Stores the curent position of the UI
        **********************************************************************/
        on select_object_rollout moved pos do
            setIniSetting select_objects_ini "Dialog" "Position" (pos as string)
        
        /**********************************************************************
        Select material event handler. Selects all objects with a certain material class
        **********************************************************************/
        on select_material_button pressed do
        (
            if material_dropdown.selected  == undefined do
            (
                status_label.text = "ERROR: No material classes loaded"
                return false
            ) --end if
            
            local matches = selector.get_by_material material_dropdown.selected
            
            if visible_only_checkbox.checked do
                matches = selector.get_visible_objects matches

            if add_selection_checkbox.checked then
                selectMore matches
            else
            (
                if matches.count > 0 then
                    select matches
                else
                    clearSelection()
            ) --end else
            
            status_label.text = "INFO: Found " + (matches.count as string) + " object(s) with material"
        ) --end on
        
        /**********************************************************************
        Select modifier event handler. Selects all objects with a certain modifier class
        **********************************************************************/
        on select_modifier_button pressed do
        (
            if modifier_dropdown.selected  == undefined do
            (
                status_label.text = "ERROR: No modifier classes loaded"
                return false
            ) --end if
            
            local matches = selector.get_by_modifier modifier_dropdown.selected
            
            if visible_only_checkbox.checked do
                matches = selector.get_visible_objects matches

            if add_selection_checkbox.checked then
                selectMore matches
            else
            (
                if matches.count > 0 then
                    select matches
                else
                    clearSelection()
            ) --end else
            
            status_label.text = "INFO: Found " + (matches.count as string) + " object(s) with modifier"
        ) --end on
        
        /**********************************************************************
        Parent checkbox changed event handler. Enables/disables the immediate only checkbox
        **********************************************************************/
        on parent_checkbox changed the_state do
            extended_parent_checkbox.enabled = the_state
        
        /**********************************************************************
        Children checkbox changed event handler. Enables/disables the immediate only checkbox
        **********************************************************************/
        on children_checkbox changed the_state do
            extended_children_checkbox.enabled = the_state
        
        /**********************************************************************
        Select hierarchy event handler. Selects parents/children of the current selection
        **********************************************************************/
        on select_hierarchy_button pressed do
        (
            if not parent_checkbox.checked and not children_checkbox.checked do
            (
                status_label.text = "ERROR: Parents and/or children must be enabled"
                return false
            ) --end if
            
            if selection.count == 0 do
            (
                status_label.text = "ERROR: Nothing is selected"
                return false
            ) --end if
            
            local matches = #()
            
            if parent_checkbox.checked do
                join matches (selector.get_parents extended_parent_checkbox.checked)
            
            if children_checkbox.checked do
                join matches (selector.get_children extended_children_checkbox.checked)
            
            if visible_only_checkbox.checked do
                matches = selector.get_visible_objects matches

            if add_selection_checkbox.checked then
                selectMore matches
            else
            (
                if matches.count > 0 then
                    select matches
                else
                    clearSelection()
            ) --end else
            
            status_label.text = "INFO: Found " + (matches.count as string) + " related object(s)"
        ) --end on
        
        /**********************************************************************
        Select animation event handler. Selects objects that are animated
        **********************************************************************/
        on select_animation_button pressed do
        (
            local matches = selector.get_by_animation()
            
            if visible_only_checkbox.checked do
                matches = selector.get_visible_objects matches

            if add_selection_checkbox.checked then
                selectMore matches
            else
            (
                if matches.count > 0 then
                    select matches
                else
                    clearSelection()
            ) --end else
            
            status_label.text = "INFO: Found " + (matches.count as string) + " animated object(s)"
        ) --end on
        
        /**********************************************************************
        Select no UV event handler. Selects objects that have no UVs
        **********************************************************************/
        on select_no_uv_button pressed do
        (
            local matches = selector.get_no_uv_geometry()
            
            if visible_only_checkbox.checked do
                matches = selector.get_visible_objects matches

            if add_selection_checkbox.checked then
                selectMore matches
            else
            (
                if matches.count > 0 then
                    select matches
                else
                    clearSelection()
            ) --end else
            
            status_label.text = "INFO: Found " + (matches.count as string) + " object(s) without UVs"
        ) --end on
    ) --end rollout
    
    select_objects_position = execute (getIniSetting select_objects_ini "Dialog" "Position")
    if select_objects_position == OK do select_objects_position = [100,100]
    if select_objects_position.x > sysinfo.desktopsize.x - 250 do
        select_objects_position.x = sysinfo.desktopsize.x - 255
    if select_objects_position.y > sysinfo.desktopsize.y - 470 do
        select_objects_position.y = sysinfo.desktopsize.y - 470
    if select_objects_position.x < 0 do select_objects_position.x = 0 
    if select_objects_position.y < 0 do select_objects_position.y = 0
    createDialog select_object_rollout 250 410 select_objects_position.x select_objects_position.y
) --end local scope
